Schematics — Schematics 0.9.2 documentation

### Navigation

- index
- modules |
- next |
- Schematics 0.9.2 documentation »

# Schematics

Python Data Structures for Humans™.

For more information, please see our documentation: http://schematics.readthedocs.org/en/latest/

## About

Schematics is a Python library to combine types into structures, validate them, and transform the shapes of your data based on simple descriptions.

The internals are similar to ORM type systems, but there is no database layer in Schematics. Instead, we believe that building a database layer is made significantly easier when Schematics handles everything but writing the query.

Further, it can be used for a range of tasks where having a database involved may not make sense.

Some common use cases:

- Design and document specific data structures
- Convert structures to and from different formats such as JSON or MsgPack
- Validate API inputs
- Remove fields based on access rights of some data’s recipient
- Define message formats for communications protocols, like an RPC
- Custom persistence layers

### Examples

This is a simple Model.

    >>> from schematics.models import Model
    >>> from schematics.types import StringType, URLType
    >>> class Person(Model):
    ...     name = StringType(required=True)
    ...     website = URLType()
    ...
    >>> person = Person({'name': u'Joe Strummer',
    ...                  'website': 'http://soundcloud.com/joestrummer'})
    >>> person.name
    u'Joe Strummer'

Serializing the data to JSON.

    >>> import json
    >>> json.dumps(person.to_primitive())
    {"name": "Joe Strummer", "website": "http://soundcloud.com/joestrummer"}

Let’s try validating without a name value, since it’s required.

    >>> person = Person()
    >>> person.website = 'http://www.amontobin.com/'
    >>> person.validate()
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
      File "schematics/models.py", line 231, in validate
        raise ModelValidationError(e.messages)
    schematics.exceptions.ModelValidationError: {'name': [u'This field is required.']}

Add the field and validation passes

    >>> person = Person()
    >>> person.name = 'Amon Tobin'
    >>> person.website = 'http://www.amontobin.com/'
    >>> person.validate()
    >>>

## What’s with the fork?

At the top of this projects Github page is says “forked from exfm/dictshield”. James (@j2labs) started dictshield while working for exfm. It was open sourced, so he forked it and continued work on it.

Alas, the name, which was originally a 3am decision to make me James laugh turned into something that was awkward and a little crude, so it was renamed Schematics.

DictShield still exists, but consider anything with that label to be a ghost from this project’s early years.

## Installing

Install stable releases of Schematics with `pip`.

    $ pip install schematics

See the Install Guide for more detail.

## Documentation

### Getting Started

New Schematics users should start with the Quickstart Guide. That is the fastest way to get a look at what Schematics does.

### Usage

Schematics exists to make a few concepts easy to glue together. The types allow us to describe units of data, models let us put them together into structures with fields. We can then import data, check if it looks correct, and easily serialize the results into any format we need.

- Types
  - Coercion
  - Conversion
  - Validation
- Models
  - Simple Model
  - Model Configuration
- Exporting
  - Terminology
  - Converting Data
  - Compound Types
  - Customizing Output
- Importing
  - Compound Types
- Validation
  - Validation Errors
  - Extending Validation

### API Reference

The User’s Guide provides the high-level concepts, but the code itself provides the most accurate reference.

- Models
- Validation
- Transforms
- Types
- Contrib

## Development

We welcome ideas and code. We ask that you follow some of our guidelines though.

See the Developer’s Guide for more information.

## Community

Schematics was created in Brooklyn, NY by James Dennis. Since then, the code has been worked on by folks from around the world. If you have ideas, we encourage you to share them!

Special thanks to Hacker School, Plain Vanilla, Quantopian, Apple, Johns Hopkins University, and everyone who has contributed to Schematics.

### Bugs & Features

We track bugs, feature requests, and documentation requests with Github Issues.

### Mailing list

We discuss the future of Schematics and upcoming changes in detail on schematics-dev.

If you’ve read the documentation and still haven’t found the answer you’re looking for, you should reach out to us here too.

### Contributing

If you’re interested in contributing code or documentation to Schematics, please visit the Developer’s Guide for instructions.

### Navigation

- index
- modules |
- next |
- Schematics 0.9.2 documentation »